@@ -17,18 +17,19 @@ import collections
 import copy
 import itertools
 import random
-import re
+import time
 
 from novaclient import exceptions as nova_exception
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import timeutils
+import six
 
-from ec2api.api import clients
 from ec2api.api import common
 from ec2api.api import ec2utils
 from ec2api.api import network_interface as network_interface_api
 from ec2api.api import security_group as security_group_api
+from ec2api import clients
 from ec2api import context as ec2_context
 from ec2api.db import api as db_api
 from ec2api import exception
@@ -42,8 +43,8 @@ ec2_opts = [
                 help='Return the IP address as private dns hostname in '
                      'describe instances'),
     cfg.StrOpt('default_flavor',
-           default='m1.small',
-           help='A flavor to use as a default instance type')
+               default='m1.small',
+               help='A flavor to use as a default instance type')
 ]
 
 CONF = cfg.CONF
@@ -108,16 +109,9 @@ def run_instances(context, image_id, min_count, max_count,
             context, image_id, kernel_id, ramdisk_id)
 
     nova = clients.nova(context)
-    try:
-        if instance_type is None:
-            instance_type = CONF.default_flavor
-        os_flavor = next(f for f in nova.flavors.list()
-                         if f.name == instance_type)
-    except StopIteration:
-        raise exception.InvalidParameterValue(value=instance_type,
-                                              parameter='InstanceType')
+    os_flavor = _get_os_flavor(instance_type, nova)
 
-    bdm = _parse_block_device_mapping(context, block_device_mapping)
+    bdm = _build_block_device_mapping(context, block_device_mapping, os_image)
     availability_zone = (placement or {}).get('availability_zone')
     if user_data:
         user_data = base64.b64decode(user_data)
@@ -156,7 +150,7 @@ def run_instances(context, image_id, min_count, max_count,
                 min_count=1, max_count=1,
                 kernel_id=os_kernel_id, ramdisk_id=os_ramdisk_id,
                 availability_zone=availability_zone,
-                block_device_mapping=bdm,
+                block_device_mapping_v2=bdm,
                 key_name=key_name, userdata=user_data,
                 **extra_params)
             cleaner.addCleanup(nova.servers.delete, os_instance.id)
@@ -167,6 +161,8 @@ def run_instances(context, image_id, min_count, max_count,
                         'launch_index': launch_index}
             if client_token:
                 instance['client_token'] = client_token
+            if disable_api_termination:
+                instance['disable_api_termination'] = disable_api_termination
 
             instance = db_api.add_item(context, 'i', instance)
             cleaner.addCleanup(db_api.delete_item, context, instance['id'])
@@ -194,6 +190,12 @@ def terminate_instances(context, instance_id):
 
     nova = clients.nova(context)
     state_changes = []
+    for instance in instances:
+        if instance.get('disable_api_termination'):
+            message = _("The instance '%s' may not be terminated. Modify its "
+                        "'disableApiTermination' instance attribute and try "
+                        "again.") % instance['id']
+            raise exception.OperationNotPermitted(message=message)
     for instance in instances:
         try:
             os_instance = nova.servers.get(instance['os_id'])
@@ -212,6 +214,7 @@ def terminate_instances(context, instance_id):
 class InstanceDescriber(common.TaggableItemsDescriber):
 
     KIND = 'i'
+    SORT_KEY = 'instanceId'
     FILTER_MAP = {
         'availability-zone': ('placement', 'availabilityZone'),
         'block-device-mapping.delete-on-termination': [
@@ -294,7 +297,7 @@ class InstanceDescriber(common.TaggableItemsDescriber):
         super(InstanceDescriber, self).__init__()
         self.reservations = {}
         self.reservation_instances = collections.defaultdict(list)
-        self.reservation_os_groups = {}
+        self.reservation_groups = {}
         self.obsolete_instances = []
 
     def format(self, instance, os_instance):
@@ -302,7 +305,7 @@ class InstanceDescriber(common.TaggableItemsDescriber):
                 self.context, instance, os_instance,
                 self.ec2_network_interfaces.get(instance['id']),
                 self.image_ids, self.volumes, self.os_volumes,
-                self.os_flavors)
+                self.os_flavors, self.groups_name_to_id)
 
         reservation_id = instance['reservation_id']
         if reservation_id in self.reservations:
@@ -312,11 +315,11 @@ class InstanceDescriber(common.TaggableItemsDescriber):
                            'owner_id': os_instance.tenant_id}
             self.reservations[reservation_id] = reservation
             if not instance['vpc_id']:
-                self.reservation_os_groups[reservation_id] = (
-                    getattr(os_instance, 'security_groups', []))
+                self.reservation_groups[reservation_id] = (
+                    formatted_instance.get('groupSet'))
 
         self.reservation_instances[
-                reservation['id']].append(formatted_instance)
+            reservation['id']].append(formatted_instance)
 
         return formatted_instance
 
@@ -325,6 +328,7 @@ class InstanceDescriber(common.TaggableItemsDescriber):
         self.ec2_network_interfaces = (
             instance_engine.get_ec2_network_interfaces(
                 self.context, self.ids))
+        self.groups_name_to_id = _get_groups_name_to_id(self.context)
         self.volumes = {v['os_id']: v
                         for v in db_api.get_items(self.context, 'vol')}
         self.image_ids = {i['os_id']: i['id']
@@ -337,7 +341,7 @@ class InstanceDescriber(common.TaggableItemsDescriber):
         self.os_volumes = _get_os_volumes(self.context)
         self.os_flavors = _get_os_flavors(self.context)
         nova = clients.nova(ec2_context.get_os_admin_context())
-        if self.ids == 1 and len(self.items) == 1:
+        if len(self.ids) == 1 and len(self.items) == 1:
             try:
                 return [nova.servers.get(self.items[0]['os_id'])]
             except nova_exception.NotFound:
@@ -377,12 +381,13 @@ class ReservationDescriber(common.NonOpenstackItemsDescriber):
             return None
         return _format_reservation(self.context, reservation,
                                    formatted_instances,
-                                   self.os_groups.get(reservation['id']))
+                                   self.groups.get(reservation['id'], []))
 
     def get_db_items(self):
         return self.reservations
 
-    def describe(self, context, ids=None, names=None, filter=None):
+    def describe(self, context, ids=None, names=None, filter=None,
+                 max_results=None, next_token=None):
         reservation_filters = []
         instance_filters = []
         for f in filter or []:
@@ -400,7 +405,8 @@ class ReservationDescriber(common.NonOpenstackItemsDescriber):
         try:
             instance_describer = InstanceDescriber()
             formatted_instances = instance_describer.describe(
-                    context, ids=ids, filter=instance_filters)
+                context, ids=ids, filter=instance_filters,
+                max_results=max_results, next_token=next_token)
         except exception.InvalidInstanceIDNotFound:
             _remove_instances(context, instance_describer.obsolete_instances)
             raise
@@ -409,19 +415,32 @@ class ReservationDescriber(common.NonOpenstackItemsDescriber):
 
         self.reservations = instance_describer.reservations.values()
         self.instances = instance_describer.reservation_instances
-        self.os_groups = instance_describer.reservation_os_groups
+        self.groups = instance_describer.reservation_groups
         self.suitable_instances = set(i['instanceId']
                                       for i in formatted_instances)
 
-        return super(ReservationDescriber, self).describe(
-                context, filter=reservation_filters)
+        result = super(ReservationDescriber, self).describe(
+            context, filter=reservation_filters)
+        self.next_token = instance_describer.next_token
+        return result
 
 
 def describe_instances(context, instance_id=None, filter=None,
                        max_results=None, next_token=None):
-    formatted_reservations = ReservationDescriber().describe(
-            context, ids=instance_id, filter=filter)
-    return {'reservationSet': formatted_reservations}
+    if instance_id and max_results:
+        msg = _('The parameter instancesSet cannot be used with the parameter '
+                'maxResults')
+        raise exception.InvalidParameterCombination(msg)
+
+    reservation_describer = ReservationDescriber()
+    formatted_reservations = reservation_describer.describe(
+            context, ids=instance_id, filter=filter,
+            max_results=max_results, next_token=next_token)
+
+    result = {'reservationSet': formatted_reservations}
+    if reservation_describer.next_token:
+        result['nextToken'] = reservation_describer.next_token
+    return result
 
 
 def reboot_instances(context, instance_id):
@@ -455,7 +474,7 @@ def get_password_data(context, instance_id):
     now = timeutils.utcnow()
     return {"instanceId": instance_id,
             "timestamp": now,
-            "passwordData": base64.b64encode(password)}
+            "passwordData": base64.b64encode(password.encode())}
 
 
 def get_console_output(context, instance_id):
@@ -468,7 +487,7 @@ def get_console_output(context, instance_id):
     now = timeutils.utcnow()
     return {"instanceId": instance_id,
             "timestamp": now,
-            "output": base64.b64encode(console_output)}
+            "output": base64.b64encode(console_output.encode())}
 
 
 def describe_instance_attribute(context, instance_id, attribute):
@@ -481,13 +500,36 @@ def describe_instance_attribute(context, instance_id, attribute):
         # but AWS doesn't. This is legacy behavior of Nova EC2
         _cloud_format_instance_bdm(context, os_instance, result)
 
+    def _format_source_dest_check(result):
+        if not instance.get('vpc_id'):
+            raise exception.InvalidParameterCombination(
+                _('You may only describe the sourceDestCheck attribute for '
+                  'VPC instances'))
+        enis = network_interface_api.describe_network_interfaces(
+            context, filter=[{'name': 'attachment.instance-id',
+                              'value': [instance_id]}]
+        )['networkInterfaceSet']
+        if len(enis) != 1:
+            raise exception.InvalidInstanceId(instance_id=instance_id)
+        result['sourceDestCheck'] = {'value': enis[0]['sourceDestCheck']}
+
     def _format_attr_group_set(result):
-        result['groupSet'] = _format_group_set(
-            context, getattr(os_instance, 'security_groups', []))
+        if instance.get('vpc_id'):
+            enis = network_interface_api.describe_network_interfaces(
+                context, filter=[{'name': 'attachment.instance-id',
+                                  'value': [instance_id]}]
+            )['networkInterfaceSet']
+            if len(enis) != 1:
+                raise exception.InvalidInstanceId(instance_id=instance_id)
+            result['groupSet'] = enis[0]['groupSet']
+        else:
+            groups = _get_groups_name_to_id(context)
+            result['groupSet'] = _format_group_set(
+                context, getattr(os_instance, 'security_groups', []), groups)
 
     def _format_attr_instance_type(result):
-        result['instanceType'] = {'value': _cloud_format_instance_type(
-                                                       context, os_instance)}
+        result['instanceType'] = {
+            'value': _cloud_format_instance_type(context, os_instance)}
 
     def _format_attr_kernel(result):
         value = _cloud_format_kernel_id(context, os_instance)
@@ -503,20 +545,19 @@ def describe_instance_attribute(context, instance_id, attribute):
                                  'OS-EXT-SRV-ATTR:root_device_name', None)}
 
     def _format_attr_user_data(result):
-        if not hasattr(os_instance, 'OS-EXT-SRV-ATTR:user_data'):
-            # NOTE(ft): partial compatibility with pre Kilo OS releases
-            raise exception.InvalidAttribute(attr=attribute)
-        user_data = getattr(os_instance, 'OS-EXT-SRV-ATTR:user_data')
+        user_data = getattr(os_instance, 'OS-EXT-SRV-ATTR:user_data', None)
         if user_data:
             result['userData'] = {'value': user_data}
 
     def _format_attr_disable_api_termination(result):
-        result['disableApiTermination'] = {'value': False}
+        result['disableApiTermination'] = {
+            'value': instance.get('disable_api_termination', False)}
 
     attribute_formatter = {
         'blockDeviceMapping': _format_attr_block_device_mapping,
         'disableApiTermination': _format_attr_disable_api_termination,
         'groupSet': _format_attr_group_set,
+        'sourceDestCheck': _format_source_dest_check,
         'instanceType': _format_attr_instance_type,
         'kernel': _format_attr_kernel,
         'ramdisk': _format_attr_ramdisk,
@@ -526,28 +567,170 @@ def describe_instance_attribute(context, instance_id, attribute):
 
     fn = attribute_formatter.get(attribute)
     if fn is None:
-        # TODO(ft): clarify an exact AWS error
-        raise exception.InvalidAttribute(attr=attribute)
+        raise exception.InvalidParameterValue(value=attribute,
+                                              parameter='attribute',
+                                              reason='Unknown attribute.')
 
-    result = {'instance_id': instance_id}
+    result = {'instanceId': instance_id}
     fn(result)
     return result
 
 
-def _format_reservation(context, reservation, formatted_instances, os_groups):
+def modify_instance_attribute(context, instance_id, attribute=None,
+                              value=None, source_dest_check=None,
+                              block_device_mapping=None,
+                              disable_api_termination=None,
+                              ebs_optimized=None, group_id=None,
+                              instance_initiated_shutdown_behavior=None,
+                              instance_type=None, kernel=None,
+                              ramdisk=None, sriov_net_support=None,
+                              user_data=None):
+    # NOTE(andrey-mp): other parameters can be added in same way
+
+    if attribute is not None:
+        if attribute == 'disableApiTermination':
+            if disable_api_termination is not None:
+                raise exception.InvalidParameterCombination()
+        elif attribute == 'sourceDestCheck':
+            if source_dest_check is not None:
+                raise exception.InvalidParameterCombination()
+        elif attribute == 'instanceType':
+            if instance_type is not None:
+                raise exception.InvalidParameterCombination()
+        else:
+            raise exception.InvalidParameterValue(value=attribute,
+                                                  parameter='attribute',
+                                                  reason='Unknown attribute.')
+        if value is None:
+            raise exception.MissingParameter(param='value')
+
+    params_count = (
+        int(source_dest_check is not None) +
+        int(group_id is not None) + int(instance_type is not None) +
+        int(disable_api_termination is not None))
+    if (params_count > 1 or
+            (attribute is not None and params_count == 1) or
+            (params_count == 0 and attribute is None)):
+        raise exception.InvalidParameterCombination()
+
+    if attribute == 'disableApiTermination':
+        disable_api_termination = value
+    elif attribute == 'sourceDestCheck':
+        source_dest_check = value
+    elif attribute == 'instanceType':
+        instance_type = value
+
+    instance = ec2utils.get_db_item(context, instance_id)
+    if disable_api_termination is not None:
+        instance['disable_api_termination'] = value
+        db_api.update_item(context, instance)
+        return True
+    elif group_id is not None:
+        _modify_group(context, instance, group_id)
+        return True
+    elif source_dest_check is not None:
+        _modify_source_dest_check(context, instance, source_dest_check)
+        return True
+    elif instance_type:
+        _modify_instance_type(context, instance, instance_type)
+        return True
+
+    raise exception.InvalidParameterCombination()
+
+
+def _modify_group(context, instance, group_id):
+    if not instance.get('vpc_id'):
+        raise exception.InvalidParameterCombination(
+            _('You may only modify the groupSet attribute for VPC instances'))
+    enis = network_interface_api.describe_network_interfaces(
+        context, filter=[{'name': 'attachment.instance-id',
+                          'value': [instance['id']]}]
+    )['networkInterfaceSet']
+    if len(enis) != 1:
+        raise exception.InvalidInstanceId(instance_id=instance['id'])
+    network_interface_api.modify_network_interface_attribute(
+        context, enis[0]['networkInterfaceId'], security_group_id=group_id)
+
+
+def _modify_source_dest_check(context, instance, source_dest_check):
+    if not instance.get('vpc_id'):
+        raise exception.InvalidParameterCombination(
+            _('You may  only modify the sourceDestCheck attribute for '
+              'VPC instances'))
+    enis = network_interface_api.describe_network_interfaces(
+        context, filter=[{'name': 'attachment.instance-id',
+                          'value': [instance['id']]}]
+    )['networkInterfaceSet']
+    if len(enis) != 1:
+        raise exception.InvalidInstanceId(instance_id=instance['id'])
+    network_interface_api.modify_network_interface_attribute(
+        context, enis[0]['networkInterfaceId'],
+        source_dest_check=source_dest_check)
+
+
+def _modify_instance_type(context, instance, instance_type):
+    nova = clients.nova(context)
+    os_instance = nova.servers.get(instance['os_id'])
+    os_flavor = _get_os_flavor(instance_type, nova)
+    vm_state = getattr(os_instance, 'OS-EXT-STS:vm_state')
+    if vm_state != vm_states_STOPPED:
+        msg = (_("The instance %s is not in the 'stopped' state.")
+               % instance['id'])
+        raise exception.IncorrectInstanceState(message=msg)
+
+    if os_instance.flavor['id'] == os_flavor.id:
+        return True
+
+    os_instance.resize(os_flavor)
+    # NOTE(andrey-mp): if this operation will be too long (more than
+    # timeout) then we can add more code. For example:
+    # 1. current code returns HTTP 500 code if time is out. client retries
+    # query. code can detect that resizing in progress and wait again.
+    # 2. make this operation async by some way...
+    for dummy in range(45):
+        os_instance = nova.servers.get(os_instance)
+        vm_state = getattr(os_instance, 'OS-EXT-STS:vm_state')
+        if vm_state == vm_states_RESIZED:
+            break
+        time.sleep(1)
+    os_instance = nova.servers.get(os_instance)
+    vm_state = getattr(os_instance, 'OS-EXT-STS:vm_state')
+    if vm_state != vm_states_RESIZED:
+        raise exception.EC2APIException(
+            message=_('Time is out for instance resizing'))
+    os_instance.confirm_resize()
+    for dummy in range(15):
+        os_instance = nova.servers.get(os_instance)
+        vm_state = getattr(os_instance, 'OS-EXT-STS:vm_state')
+        if vm_state != vm_states_RESIZED:
+            break
+        time.sleep(1)
+
+
+def reset_instance_attribute(context, instance_id, attribute):
+    if attribute == 'sourceDestCheck':
+        instance = ec2utils.get_db_item(context, instance_id)
+        _modify_source_dest_check(context, instance, True)
+        return True
+
+    raise exception.InvalidParameterValue(value=attribute,
+                                          parameter='attribute',
+                                          reason='Unknown attribute.')
+
+
+def _format_reservation(context, reservation, formatted_instances, groups):
     return {
         'reservationId': reservation['id'],
         'ownerId': reservation['owner_id'],
         'instancesSet': sorted(formatted_instances,
                                key=lambda i: i['amiLaunchIndex']),
-        'groupSet': (_format_group_set(context, os_groups)
-                     if os_groups is not None else [])
+        'groupSet': groups
     }
 
 
 def _format_instance(context, instance, os_instance, ec2_network_interfaces,
-                     image_ids, volumes=None, os_volumes=None,
-                     os_flavors=None):
+                     image_ids, volumes, os_volumes, os_flavors,
+                     groups_name_to_id):
     ec2_instance = {
         'amiLaunchIndex': instance['launch_index'],
         'imageId': (ec2utils.os_id_to_ec2_id(context, 'ami',
@@ -587,10 +770,9 @@ def _format_instance(context, instance, os_instance, ec2_network_interfaces,
         if fixed_ip6:
             ec2_instance['dnsNameV6'] = fixed_ip6
         dns_name = floating_ip
-        # TODO(ft): euca2ools require groupId for an instance security group.
-        # But ec2-api doesn't store IDs for EC2 Classic groups.
-        # ec2_instance['groupSet'] = _format_group_set(
-        #         context, os_instance.security_groups)
+        if getattr(os_instance, 'security_groups', None):
+            ec2_instance['groupSet'] = _format_group_set(
+                context, os_instance.security_groups, groups_name_to_id)
     else:
         primary_ec2_network_interface = None
         for ec2_network_interface in ec2_network_interfaces:
@@ -615,7 +797,9 @@ def _format_instance(context, instance, os_instance, ec2_network_interfaces,
         if primary_ec2_network_interface:
             ec2_instance.update({
                 'subnetId': primary_ec2_network_interface['subnetId'],
-                'groupSet': primary_ec2_network_interface['groupSet']})
+                'groupSet': primary_ec2_network_interface['groupSet'],
+                'sourceDestCheck':
+                    primary_ec2_network_interface['sourceDestCheck']})
             fixed_ip = primary_ec2_network_interface['privateIpAddress']
             if 'association' in primary_ec2_network_interface:
                 association = primary_ec2_network_interface['association']
@@ -681,10 +865,6 @@ def _remove_instances(context, instances):
 
 
 def _check_min_max_count(min_count, max_count):
-    # TODO(ft): figure out appropriate aws message and use them
-    min_count = int(min_count)
-    max_count = int(max_count)
-
     if min_count < 1:
         msg = _('Minimum instance count must be greater than zero')
         raise exception.InvalidParameterValue(msg)
@@ -714,29 +894,162 @@ def _parse_image_parameters(context, image_id, kernel_id, ramdisk_id):
 
 def _parse_block_device_mapping(context, block_device_mapping):
     # TODO(ft): check block_device_mapping structure
-    bdm = {}
+    # TODO(ft): support virtual devices
+    # TODO(ft): support no_device
+    bdms = []
     for args_bd in (block_device_mapping or []):
-        _cloud_parse_block_device_mapping(context, args_bd)
-        bdm[args_bd['device_name']] = ':'.join(
-            [args_bd.get('snapshot_id', args_bd.get('volume_id', '')),
-             ('snap' if 'snapshot_id' in args_bd else
-              'vol' if 'volume_id' in args_bd else ''),
-             str(args_bd.get('volume_size', '')),
-             str(args_bd.get('delete_on_termination', ''))])
-    return bdm
+        bdm = {
+            'device_name': args_bd['device_name'],
+            'destination_type': 'volume',
+        }
+
+        ebs = args_bd.get('ebs')
+        if ebs:
+            ec2_id = ebs.get('snapshot_id')
+            if ec2_id:
+                if ec2_id.startswith('snap-'):
+                    bdm['source_type'] = 'snapshot'
+                    snapshot = ec2utils.get_db_item(context, ec2_id)
+                    bdm['snapshot_id'] = snapshot['os_id']
+                # NOTE(ft): OpenStack extension, AWS incompatibility
+                elif ec2_id.startswith('vol-'):
+                    bdm['source_type'] = 'volume'
+                    volume = ec2utils.get_db_item(context, ec2_id)
+                    bdm['volume_id'] = volume['os_id']
+                else:
+                    # NOTE(ft): AWS returns undocumented
+                    # InvalidSnapshotID.NotFound
+                    raise exception.InvalidSnapshotIDMalformed(
+                        snapshot_id=ec2_id)
+            if 'volume_size' in ebs:
+                bdm['volume_size'] = ebs['volume_size']
+            if 'delete_on_termination' in ebs:
+                bdm['delete_on_termination'] = ebs['delete_on_termination']
+
+        # substitute a previous bdm which has the same device name
+        short_device_name = ec2utils.block_device_strip_dev(bdm['device_name'])
+        first_bdm, index = next(
+            ((m, i) for i, m in enumerate(bdms)
+             if (ec2utils.block_device_strip_dev(m['device_name']) ==
+                 short_device_name)),
+            (None, None))
+        if first_bdm:
+            if bdm['device_name'] == first_bdm['device_name']:
+                bdms.pop(index)
+            else:
+                msg = _("The device '%s' is used in more than one "
+                        "block-device mapping") % short_device_name
+                raise exception.InvalidBlockDeviceMapping(msg)
+
+        bdms.append(bdm)
+
+    return bdms
 
 
-def _format_group_set(context, os_security_groups):
+def _build_block_device_mapping(context, block_device_mapping, os_image):
+    mappings = _parse_block_device_mapping(context, block_device_mapping)
+    properties = ec2utils.deserialize_os_image_properties(os_image)
+    image_bdms = ec2utils.get_os_image_mappings(properties)
+    root_device_name = (
+        ec2utils.block_device_properties_root_device_name(properties))
+    short_root_device_name = ec2utils.block_device_strip_dev(root_device_name)
+
+    # build a dict of image bmds to make the merge easier
+    # set some default values to a root bdm to simplify checks in mapping loop
+    image_bdm_dict = {}
+    for bdm in image_bdms:
+        if bdm.get('device_name'):
+            key = ec2utils.block_device_strip_dev(bdm['device_name'])
+            if key == short_root_device_name:
+                bdm.setdefault('boot_index', 0)
+        elif bdm.get('boot_index') == 0:
+            key = short_root_device_name
+            bdm.setdefault('device_name', root_device_name)
+        else:
+            continue
+        image_bdm_dict[key] = bdm
+    result = []
+
+    # convert mappings to be ready to pass in nova.servers.create
+    # and merge to them a corresponding image bdm if existing
+    # (because Nova only supports an overloading, but not the merging)
+    for bdm in mappings:
+        short_device_name = ec2utils.block_device_strip_dev(bdm['device_name'])
+        if short_device_name not in image_bdm_dict:
+            _populate_parsed_bdm_parameter(bdm, short_root_device_name)
+        else:
+            image_bdm = image_bdm_dict[short_device_name]
+            if bdm['device_name'] != image_bdm['device_name']:
+                raise exception.InvalidBlockDeviceMapping(
+                    _("The device '%s' is used in more than one "
+                      "block-device mapping") % short_device_name)
+            if (image_bdm.get('boot_index') == 0 and 'snapshot_id' in bdm and
+                    bdm['snapshot_id'] != image_bdm.get('snapshot_id')):
+                raise exception.InvalidBlockDeviceMapping(
+                    _('snapshotId cannot be modified on root device'))
+            if ('volume_size' in bdm and 'volume_size' in image_bdm and
+                    bdm['volume_size'] < image_bdm['volume_size']):
+                raise exception.InvalidBlockDeviceMapping(
+                    _("Volume of size %(bdm_size)dGB is smaller than expected "
+                      "size %(image_bdm_size)dGB for '(device_name)s'") %
+                    {'bdm_size': bdm['volume_size'],
+                     'image_bdm_size': image_bdm['volume_size'],
+                     'device_name': bdm['device_name']})
+
+            if bdm.get('snapshot_id'):
+                if 'snapshot_id' not in image_bdm:
+                    raise exception.InvalidBlockDeviceMapping(
+                        _('snapshotId can only be modified on EBS devices'))
+
+                _populate_parsed_bdm_parameter(bdm, short_root_device_name)
+            else:
+                image_bdm.update(bdm)
+                bdm = image_bdm
+
+        # move source id to nova.servers.create related parameter
+        # NOTE(ft): safely extract source id, because we do not validate
+        # v2 image bdm, thus the bdm may be invalid and do not contain
+        # mandatory keys
+        source_type = bdm.get('source_type')
+        if source_type and source_type != 'blank':
+            uuid = bdm.pop('_'.join([source_type, 'id']), None)
+            bdm['uuid'] = uuid
+
+        result.append(bdm)
+
+    return result
+
+
+def _populate_parsed_bdm_parameter(bdm, short_root_device_name):
+    bdm.setdefault('delete_on_termination', True)
+    bdm.setdefault('source_type', 'blank')
+    if (short_root_device_name ==
+            ec2utils.block_device_strip_dev(bdm['device_name'])):
+        bdm['boot_index'] = 0
+    else:
+        bdm['boot_index'] = -1
+
+
+def _format_group_set(context, os_security_groups, groups):
     if not os_security_groups:
         return None
-    # TODO(ft): Euca tools uses 2010-08-31 AWS protocol version which doesn't
-    # contain groupId in groupSet of an instance structure
-    # Euca crashes if groupId is present here
-    return [{'groupName': sg['name']} for sg in os_security_groups]
+    return [{'groupName': sg['name'],
+             'groupId': groups[sg['name']]}
+            for sg in os_security_groups
+            if sg['name'] in groups]
+
+
+def _get_groups_name_to_id(context):
+    # TODO(andrey-mp): remove filtering by vpcId=None when fitering
+    # by None will be implemented
+    return {g['groupName']: g['groupId']
+            for g in (security_group_api.describe_security_groups(context)
+                      ['securityGroupInfo'])
+            if not g.get('vpcId')}
 
 
 def _get_ip_info_for_instance(os_instance):
-    addresses = list(itertools.chain(*os_instance.addresses.itervalues()))
+    addresses = list(itertools.chain(*six.itervalues(os_instance.addresses)))
     fixed_ip = next((addr['addr'] for addr in addresses
                      if (addr['version'] == 4 and
                          addr['OS-EXT-IPS:type'] == 'fixed')), None)
@@ -800,6 +1113,18 @@ def _get_os_volumes(context):
     return os_volumes
 
 
+def _get_os_flavor(instance_type, nova):
+    try:
+        if instance_type is None:
+            instance_type = CONF.default_flavor
+        os_flavor = next(f for f in nova.flavors.list()
+                         if f.name == instance_type)
+    except StopIteration:
+        raise exception.InvalidParameterValue(value=instance_type,
+                                              parameter='InstanceType')
+    return os_flavor
+
+
 def _is_ebs_instance(context, os_instance_id):
     nova = clients.nova(ec2_context.get_os_admin_context())
     os_instance = nova.servers.get(os_instance_id)
@@ -807,9 +1132,11 @@ def _is_ebs_instance(context, os_instance_id):
                                'OS-EXT-SRV-ATTR:root_device_name', None)
     if not root_device_name:
         return False
-    root_device_short_name = _block_device_strip_dev(root_device_name)
+    root_device_short_name = ec2utils.block_device_strip_dev(
+        root_device_name)
     if root_device_name == root_device_short_name:
-        root_device_name = _block_device_prepend_dev(root_device_name)
+        root_device_name = ec2utils.block_device_prepend_dev(
+            root_device_name)
     for os_volume in _get_os_volumes(context)[os_instance_id]:
         os_attachment = next(iter(os_volume.attachments), {})
         device_name = os_attachment.get('device')
@@ -931,8 +1258,9 @@ class InstanceEngineNeutron(object):
                  # associate_public_ip_address in all other code
                  len(network_interfaces) == 1 and
                  (len(network_interfaces[0]) != 2 or
-                     'associate_public_ip_address' not in network_interfaces[0]
-                     or network_interfaces[0].get('device_index') != 0))):
+                     ('associate_public_ip_address' not in
+                      network_interfaces[0]) or
+                     network_interfaces[0].get('device_index') != 0))):
             msg = _(' Network interfaces and an instance-level subnet ID or '
                     'private IP address or security groups may not be '
                     'specified on the same request')
@@ -1086,13 +1414,15 @@ class InstanceEngineNeutron(object):
         os_subnet_ids = [eni['os_id']
                          for eni in db_api.get_items(context, 'subnet')]
         if os_subnet_ids:
-            os_subnets = neutron.list_subnets(id=os_subnet_ids,
-                fields=['network_id'], tenant_id=context.project_id)['subnets']
+            os_subnets = neutron.list_subnets(
+                id=os_subnet_ids, fields=['network_id'],
+                tenant_id=context.project_id)['subnets']
             vpc_os_network_ids = set(sn['network_id'] for sn in os_subnets)
         else:
             vpc_os_network_ids = []
-        os_networks = neutron.list_networks(**{'router:external': False,
-            'fields': ['id'], 'tenant_id': context.project_id})['networks']
+        os_networks = neutron.list_networks(
+            **{'router:external': False, 'fields': ['id'],
+               'tenant_id': context.project_id})['networks']
         ec2_classic_os_networks = [n for n in os_networks
                                    if n['id'] not in vpc_os_network_ids]
         if len(ec2_classic_os_networks) == 0:
@@ -1151,35 +1481,6 @@ ec2utils.register_auto_create_db_item_extension(
 
 # NOTE(ft): following functions are copied from various parts of Nova
 
-def _cloud_parse_block_device_mapping(context, bdm):
-    """Parse BlockDeviceMappingItemType into flat hash
-
-    BlockDevicedMapping.<N>.DeviceName
-    BlockDevicedMapping.<N>.Ebs.SnapshotId
-    BlockDevicedMapping.<N>.Ebs.VolumeSize
-    BlockDevicedMapping.<N>.Ebs.DeleteOnTermination
-    BlockDevicedMapping.<N>.Ebs.NoDevice
-    BlockDevicedMapping.<N>.VirtualName
-    => remove .Ebs and allow volume id in SnapshotId
-    """
-    ebs = bdm.pop('ebs', None)
-    if ebs:
-        ec2_id = ebs.pop('snapshot_id', None)
-        if ec2_id:
-            if ec2_id.startswith('snap-'):
-                snapshot = ec2utils.get_db_item(context, ec2_id)
-                bdm['snapshot_id'] = snapshot['os_id']
-            elif ec2_id.startswith('vol-'):
-                volume = ec2utils.get_db_item(context, ec2_id)
-                bdm['volume_id'] = volume['os_id']
-            else:
-                # NOTE(ft): AWS returns undocumented InvalidSnapshotID.NotFound
-                raise exception.InvalidSnapshotIDMalformed(snapshot_id=ec2_id)
-            ebs.setdefault('delete_on_termination', True)
-        bdm.update(ebs)
-    return bdm
-
-
 def _cloud_get_image_state(image):
     state = image.status
     if state == 'active':
@@ -1227,9 +1528,11 @@ def _cloud_format_instance_bdm(context, os_instance, result,
         root_device_short_name = root_device_type = None
     else:
         root_device_type = 'instance-store'
-        root_device_short_name = _block_device_strip_dev(root_device_name)
+        root_device_short_name = ec2utils.block_device_strip_dev(
+            root_device_name)
         if root_device_name == root_device_short_name:
-            root_device_name = _block_device_prepend_dev(root_device_name)
+            root_device_name = ec2utils.block_device_prepend_dev(
+                root_device_name)
     mapping = []
     if os_volumes is None:
         os_volumes = _get_os_volumes(context)
@@ -1276,22 +1579,9 @@ def _cloud_get_volume_attach_status(volume):
         return 'detached'
 
 
-_dev = re.compile('^/dev/')
-
-
-def _block_device_strip_dev(device_name):
-    """remove leading '/dev/'."""
-    return _dev.sub('', device_name) if device_name else device_name
-
-
-def _block_device_prepend_dev(device_name):
-    """Make sure there is a leading '/dev/'."""
-    return device_name and '/dev/' + _block_device_strip_dev(device_name)
-
-
 def _utils_generate_uid(topic, size=8):
     characters = '01234567890abcdefghijklmnopqrstuvwxyz'
-    choices = [random.choice(characters) for _x in xrange(size)]
+    choices = [random.choice(characters) for _x in range(size)]
     return '%s-%s' % (topic, ''.join(choices))
 
 
@@ -1385,9 +1675,9 @@ _NAME_TO_CODE = {
     inst_state_SUSPEND: inst_state_STOPPED_CODE,
     inst_state_RESCUE: inst_state_RUNNING_CODE,
 }
-_CODE_TO_NAMES = {code: [item[0] for item in _NAME_TO_CODE.iteritems()
+_CODE_TO_NAMES = {code: [item[0] for item in six.iteritems(_NAME_TO_CODE)
                          if item[1] == code]
-                  for code in set(_NAME_TO_CODE.itervalues())}
+                  for code in set(six.itervalues(_NAME_TO_CODE))}
 
 
 def inst_state_name_to_code(name):
