@@ -29,6 +29,7 @@ from sqlalchemy.sql import bindparam
 
 import ec2api.context
 from ec2api.db.sqlalchemy import models
+from ec2api import exception
 
 CONF = cfg.CONF
 
@@ -83,20 +84,18 @@ def model_query(context, model, *args, **kwargs):
     return session.query(model, *args)
 
 
-def _new_id(kind, os_id):
+def _new_id(kind):
     obj_id = "%(kind)s-%(id)08x" % {"kind": kind,
                                     "id": random.randint(1, 0xffffffff)}
     return obj_id
 
 
 @require_context
-def add_item(context, kind, data, project_id=None):
-    if not project_id:
-        project_id = context.project_id
+def add_item(context, kind, data):
     item_ref = models.Item()
     item_ref.update({
-        "project_id": project_id,
-        "id": _new_id(kind, data.get("os_id")),
+        "project_id": context.project_id,
+        "id": _new_id(kind),
     })
     item_ref.update(_pack_item_data(data))
     try:
@@ -107,14 +106,14 @@ def add_item(context, kind, data, project_id=None):
             raise
         item_ref = (model_query(context, models.Item).
                     filter_by(os_id=data["os_id"]).
-                    filter(or_(models.Item.project_id == project_id,
+                    filter(or_(models.Item.project_id == context.project_id,
                                models.Item.project_id.is_(None))).
                     filter(models.Item.id.like('%s-%%' % kind)).
                     one())
         item_data = _unpack_item_data(item_ref)
         item_data.update(data)
         item_ref.update(_pack_item_data(item_data))
-        item_ref.project_id = project_id
+        item_ref.project_id = context.project_id
         item_ref.save()
     return _unpack_item_data(item_ref)
 
@@ -123,7 +122,7 @@ def add_item(context, kind, data, project_id=None):
 def add_item_id(context, kind, os_id, project_id=None):
     item_ref = models.Item()
     item_ref.update({
-        "id": _new_id(kind, os_id),
+        "id": _new_id(kind),
         "os_id": os_id,
     })
     if project_id:
@@ -144,8 +143,12 @@ def add_item_id(context, kind, os_id, project_id=None):
 def update_item(context, item):
     item_ref = (model_query(context, models.Item).
                 filter_by(project_id=context.project_id,
-                          id=item["id"]).
+                          id=item['id']).
                 one())
+    if item_ref.os_id and item_ref.os_id != item['os_id']:
+        raise exception.EC2DBInvalidOsIdUpdate(item_id=item['id'],
+                                               old_os_id=item_ref.os_id,
+                                               new_os_id=item['os_id'])
     item_ref.update(_pack_item_data(item))
     item_ref.save()
     return _unpack_item_data(item_ref)
