@@ -61,11 +61,11 @@ class EC2APIException(Exception):
                 # log the issue and the kwargs
                 LOG.exception(_('Exception in string format operation for '
                                 '%s exception'), self.__class__.__name__)
-                for name, value in kwargs.iteritems():
+                for name, value in six.iteritems(kwargs):
                     LOG.error('%s: %s' % (name, value))
 
                 if CONF.fatal_exception_format_errors:
-                    raise exc_info[0], exc_info[1], exc_info[2]
+                    six.reraise(*exc_info)
                 else:
                     # at least get the core message out if something happened
                     message = self.msg_fmt
@@ -100,6 +100,11 @@ class EC2KeystoneDiscoverFailure(EC2APIException):
     msg_fmt = _("Could not discover keystone versions.")
 
 
+class EC2DBInvalidOsIdUpdate(EC2APIException):
+    msg_fmt = _('Invalid update of os_id of %(item_id)s item '
+                'from %(old_os_id)s to %(new_os_id)s')
+
+
 # Internal ec2api metadata exceptions
 
 class EC2MetadataException(EC2APIException):
@@ -215,6 +220,11 @@ class InvalidVpcRange(EC2InvalidException):
     msg_fmt = _("The CIDR '%(cidr_block)s' is invalid.")
 
 
+class InvalidVpcState(EC2InvalidException):
+    msg_fmt = _('VPC %(vpc_id)s is currently attached to '
+                'the Virtual Private Gateway %(vgw_id)s')
+
+
 class InvalidSubnetRange(EC2InvalidException):
     ec2_code = 'InvalidSubnet.Range'
     msg_fmt = _("The CIDR '%(cidr_block)s' is invalid.")
@@ -238,6 +248,10 @@ class InvalidSnapshotIDMalformed(EC2InvalidException):
     msg_fmg = _('The snapshot %(id)s ID is not valid')
 
 
+class InvalidBlockDeviceMapping(EC2InvalidException):
+    pass
+
+
 class IncorrectState(EC2IncorrectStateException):
     msg_fmt = _("The resource is in incorrect state for the request - reason: "
                 "'%(reason)s'")
@@ -257,7 +271,7 @@ class ResourceAlreadyAssociated(EC2IncorrectStateException):
 
 class GatewayNotAttached(EC2IncorrectStateException):
     ec2_code = 'Gateway.NotAttached'
-    msg_fmt = _("resource %(igw_id)s is not attached to network %(vpc_id)s")
+    msg_fmt = _("resource %(gw_id)s is not attached to network %(vpc_id)s")
 
 
 class IncorrectInstanceState(EC2IncorrectStateException):
@@ -301,6 +315,11 @@ class RouteAlreadyExists(EC2DuplicateException):
                 'already exists.')
 
 
+class InvalidCustomerGatewayDuplicateIpAddress(EC2DuplicateException):
+    ec2_code = 'InvalidCustomerGateway.DuplicateIpAddress'
+    msg_fmt = _('Conflict among chosen gateway IP addresses.')
+
+
 class InvalidVpcIDNotFound(EC2NotFoundException):
     ec2_code = 'InvalidVpcID.NotFound'
     msg_fmt = _("The vpc ID '%(id)s' does not exist")
@@ -404,6 +423,32 @@ class InvalidAvailabilityZoneNotFound(EC2NotFoundException):
     msg_fmt = _("Availability zone %(id)s not found")
 
 
+class InvalidGatewayIDNotFound(EC2NotFoundException):
+    ec2_code = 'InvalidGatewayID.NotFound'
+    msg_fmt = _("The gateway ID '%(id)s' does not exist")
+
+
+class InvalidVpnGatewayIDNotFound(EC2NotFoundException):
+    ec2_code = 'InvalidVpnGatewayID.NotFound'
+    msg_fmt = _("The vpnGateway ID '%(id)s' does not exist")
+
+
+class InvalidCustomerGatewayIDNotFound(EC2NotFoundException):
+    ec2_code = 'InvalidCustomerGatewayID.NotFound'
+    msg_fmt = _("The customerGateway ID '%(id)s' does not exist")
+
+
+class InvalidVpnConnectionIDNotFound(EC2NotFoundException):
+    ec2_code = 'InvalidVpnConnectionID.NotFound'
+    msg_fmt = _("The vpnConnection ID '%(id)s' does not exist")
+
+
+class InvalidVpnGatewayAttachmentNotFound(EC2NotFoundException):
+    ec2_code = 'InvalidVpnGatewayAttachment.NotFound'
+    msg_fmt = _("The attachment with vpn gateway ID '%(vgw_id)s' "
+                "and vpc ID '%(vpc_id)s' does not exist")
+
+
 class ResourceLimitExceeded(EC2OverlimitException):
     msg_fmt = _('You have reached the limit of %(resource)s')
 
@@ -417,9 +462,9 @@ class SubnetLimitExceeded(EC2OverlimitException):
                 'can create')
 
 
-class NetworkInterfaceLimitExceeded(EC2OverlimitException):
-    msg_fmt = _('You have reached the limit of network interfaces for subnet'
-                '%(subnet_id)s.')
+class InsufficientFreeAddressesInSubnet(EC2OverlimitException):
+    msg_fmt = _('The specified subnet does not have enough free addresses to '
+                'satisfy the request.')
 
 
 class AddressLimitExceeded(EC2OverlimitException):
@@ -433,3 +478,13 @@ class SecurityGroupLimitExceeded(EC2OverlimitException):
 class RulesPerSecurityGroupLimitExceeded(EC2OverlimitException):
     msg_fmt = _("You've reached the limit on the number of rules that "
                 "you can add to a security group.")
+
+
+class VpnGatewayAttachmentLimitExceeded(EC2OverlimitException):
+    msg_fmt = _('The maximum number of virtual private gateway attachments '
+                'has been reached.')
+
+
+class InvalidGroupReserved(EC2InvalidException):
+    ec2_code = 'InvalidGroup.Reserved'
+    msg_fmt = _("The security group '%(group_name)' is reserved.")
