@@ -12,52 +12,25 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from oslo_config import cfg
-from oslotest import base as test_base
+import mock
 from sqlalchemy.orm import exc as orm_exception
 
 from ec2api.api import validator
-from ec2api import config
-from ec2api import context as ec2_context
 from ec2api.db import api as db_api
-from ec2api.db import migration
-from ec2api.db.sqlalchemy import api as session
+from ec2api import exception
+from ec2api.tests.unit import base
 from ec2api.tests.unit import fakes
 from ec2api.tests.unit import matchers
 
 
-class DbApiTestCase(test_base.BaseTestCase):
-
-    DB_SCHEMA = None
-
-    @classmethod
-    def setUpClass(cls):
-        super(DbApiTestCase, cls).setUpClass()
-        conf = cfg.CONF
-        try:
-            config.parse_args([], default_config_files=[])
-            conf.set_override('connection', 'sqlite://', group='database')
-            conf.set_override('sqlite_synchronous', False, group='database')
-
-            engine = session.get_engine()
-            conn = engine.connect()
-            migration.db_sync()
-            cls.DB_SCHEMA = "".join(line
-                                    for line in conn.connection.iterdump())
-            engine.dispose()
-        finally:
-            conf.reset()
+class DbApiTestCase(base.DbTestCase):
 
     def setUp(self):
         super(DbApiTestCase, self).setUp()
-        engine = session.get_engine()
-        engine.dispose()
-        conn = engine.connect()
-        conn.connection.executescript(self.DB_SCHEMA)
-        self.context = ec2_context.RequestContext(fakes.ID_OS_USER,
-                                                  fakes.ID_OS_PROJECT)
-        self.other_context = ec2_context.RequestContext(
-            fakes.random_os_id(), fakes.random_os_id())
+        self.context = mock.NonCallableMock(
+            project_id=fakes.random_os_id())
+        self.other_context = mock.NonCallableMock(
+            project_id=fakes.random_os_id())
 
     def test_add_item(self):
         new_item = {'os_id': fakes.random_os_id(),
@@ -170,6 +143,22 @@ class DbApiTestCase(test_base.BaseTestCase):
                           {'id': fakes.random_ec2_id('fake'),
                            'key': 'val'})
 
+    def test_update_item_os_id(self):
+        item = db_api.add_item(self.context, 'fake', {})
+        item['os_id'] = 'fake_os_id'
+        db_api.update_item(self.context, item)
+        item = db_api.get_item_by_id(self.context, item['id'])
+        self.assertThat({'os_id': 'fake_os_id'},
+                        matchers.IsSubDictOf(item))
+        item['os_id'] = 'other_fake_os_id'
+        self.assertRaises(exception.EC2DBInvalidOsIdUpdate,
+                          db_api.update_item,
+                          self.context, item)
+        item['os_id'] = None
+        self.assertRaises(exception.EC2DBInvalidOsIdUpdate,
+                          db_api.update_item,
+                          self.context, item)
+
     def test_delete_item(self):
         item = db_api.add_item(self.context, 'fake', {})
         db_api.delete_item(self.context, item['id'])
@@ -255,13 +244,27 @@ class DbApiTestCase(test_base.BaseTestCase):
         item = db_api.get_items(self.context, 'fake1')[0]
         other_item = db_api.get_items(self.other_context, 'fake1')[0]
         items_ids = db_api.get_items_ids(self.context, 'fake1',
-                                         [item['os_id'], other_item['os_id']])
+                                         item_os_ids=[item['os_id'],
+                                                      other_item['os_id']])
         self.assertThat(items_ids,
                         matchers.ListMatches(
                             [(item['id'], item['os_id']),
                              (other_item['id'], other_item['os_id'])],
                             orderless_lists=True))
-        items_ids = db_api.get_items_ids(self.context, 'fake', [item['os_id']])
+        items_ids = db_api.get_items_ids(self.context, 'fake',
+                                         item_os_ids=[item['os_id']])
+        self.assertEqual(0, len(items_ids))
+
+        item_ids = db_api.get_items_ids(self.context, 'fake1',
+                                        item_ids=[item['id'],
+                                                  other_item['id']])
+        self.assertThat(item_ids,
+                        matchers.ListMatches(
+                            [(item['id'], item['os_id']),
+                             (other_item['id'], other_item['os_id'])],
+                            orderless_lists=True))
+        items_ids = db_api.get_items_ids(self.context, 'fake',
+                                         item_ids=[item['id']])
         self.assertEqual(0, len(items_ids))
 
     def test_get_public_items(self):
